#include <math.h>
#include <stdio.h>

#if QT_VERSION >= 0x040000
#include <QLineEdit>
#include <QSlider>
#include <QLabel>
#else
#include <qlineedit.h>
#include <qslider.h>
#include <qlabel.h>
#endif

#include "floatSliderPlus.h"

#if QT_VERSION >= 0x040000
FloatSliderPlus::FloatSliderPlus(QWidget* parent)
	:QWidget(parent)
{
	setupUi(this) ;

	connect(valueField,SIGNAL(textChanged(const QString&)),this,SLOT(UpdateFromTextField(const QString&))) ;
	connect(    slider,SIGNAL(valueChanged(int)),this,SLOT(UpdateFromSlider(int))) ;
#else
FloatSliderPlus::FloatSliderPlus(QWidget* parent, const char* name, WFlags fl)
	:floatSliderPlus(parent,name,fl)
{
#endif
	N = width() ;
	min = 1 ;
	stuck = false ;
	max = 100 ;
	logarithmic = false ;
#if QT_VERSION >= 0x040000
	slider->setMinimum(0) ;
	slider->setMaximum(N) ;
#else
	slider->setMinValue(0) ;
	slider->setMaxValue(N) ;
#endif

	setValue(min) ;
}
	
bool FloatSliderPlus::setProperty (const char *name, const QVariant & value)
{
	bool ok = true ;

	if(!strcmp(name,"logarithmic")) 
		logarithmic = value.toBool() ;
	else if(!strcmp(name,"min")) 
		min = value.toString().toFloat(&ok) ;
	else if(!strcmp(name,"max")) 
		max = value.toString().toFloat(&ok) ;
	else if(!strcmp(name,"value")) 
		setValue(value.toString().toFloat(&ok)) ;
	if(!strcmp(name,"label")) 
		nameLabel->setText(value.toString()) ;
	else 
		return false ;

	return ok ;
}

void FloatSliderPlus::setValue(float v)
{
	if((v < min)||(v > max))
		return ;

	stuck = true ;
	valueField->setText(QString::number((double)v)) ;

	if(logarithmic)
		slider->setValue((int)floor(N*(log(v)-log(min))/(log(max)-log(min))));
	else
		slider->setValue((int)floor(N*(v-min)/(max-min)));
	stuck = false ;
}

void FloatSliderPlus::UpdateFromSlider(int v)
{
	if(stuck)
		return ;

	stuck = true ;
	float f ;

	if(logarithmic)
		f = exp(log(min) + (log(max)-log(min))*v/(float)N) ;
	else
		f = min + (max-min)*v/(float)N ;

	valueField->setText(QString::number((double)f)) ;
	stuck = false ;

	emit valueChanged(f) ;
}

void FloatSliderPlus::UpdateFromTextField(const QString & s)
{
	if(stuck)
		return ;

	bool ok ;
	stuck = true ;
	float f = s.toFloat(&ok) ;

	if(ok && (f <= max) && (f >= min))
	{
		if(logarithmic)
			slider->setValue((int)floor(N*(log(f)-log(min))/(log(max)-log(min))));
		else
			slider->setValue((int)floor(N*(f-min)/(max-min)));

		emit valueChanged(f) ;
	}
	stuck = false ;
}

void FloatSliderPlus::setValue(const QString& s)
{
	bool ok ;
	float v = s.toFloat(&ok) ;

	if((!ok)||(v < min)||(v > max))
		return ;

	stuck = true ;
	valueField->setText(s) ;
	slider->setValue((int)floor(N*(v-min)/(max-min)));
	stuck = false ;
}


