#ifndef _DATAMAT_H_
#define _DATAMAT_H_

#include <vector>
#include<iostream>
#include <iterator>
#include <map>
#include <cmath>
#include <algorithm>
#include <functional>  

using namespace std; 

typedef vector<double> DataVec ;

class DataMat
{
	public:                

		DataMat(int nRows=0, int nCols=0) ;
		DataMat(const DataMat& DM) ;
		DataMat(const DataMat& DM, int nRows, int nCols) ; // Resample DM in a nRows*nCols DataMat.
		DataMat(const string& Fname)  {Open(Fname); }
                                
		void		Open(const string& Fname) ;
		int             NCols() const                                   { return (_V.size()>0) ? _V[0].size() : 0 ; }
		int             y_size() const                                   { return _V.size(); }
		int             x_size() const                                   { return (_V.size()>0) ? _V[0].size() : 0 ; }
		int             NRows() const                                   { return _V.size(); }
		void            resize(int NR, int NC)                          {_V.resize(NR); for (int i(0); i<NR; i++) _V[i].resize(NC); }
		void            resize(const DataMat& D) ;                
		const DataMat&  Assign (const DataMat& D, double d) ;
		const DataMat&  operator += (const DataMat& D) ;
		const DataMat&  operator -= (const DataMat& D) ;
		const DataMat&  operator *= (const DataMat& D) ;
		double          GetDR() const ;
		double&         operator () (const pair<int, int>& p)           { return _V[p.first][p.second] ; }                
		const double&   operator () (const pair<int, int>& p) const     { return _V[p.first][p.second] ; }                
		const double&   operator()(int i, int j) const                  { return _V[i][j] ; }
		double&         operator()(int i, int j)                        { return _V[i][j] ; }
		
		void 		Histogram(vector<double>& hist, vector<double>& CumulHist, int nBins, bool normalize=true) const; 
                int 		StationaryPointsStatistical(vector<pair<int, int> >& mins, vector<pair<int, int> >& maxs, double Thresh, int KSize=3, int Leniency=0) const  ;
		void		BinaryWrite(const string& fname) const ;
                                
		friend void     DataMatSubtract(DataMat& Diff, const DataMat& A, const DataMat& B) ;		
		friend ostream& operator <<(ostream& os, const DataMat& Mat) ;
                
	private:
		double          Interp(double i,double j) const ;
                vector<vector<double> > _V ;
};

struct less_pair: public binary_function< const pair<int,int>&, const pair<int,int>& , bool> 
{  bool operator()(const pair<int,int>& p1, const pair<int,int>& p2) { return p1.first <= p2.first && p1.second < p2.second; } };

inline void DataMatSubtract(DataMat& Diff, const DataMat& A, const DataMat& B) 
{
	int minNR(min(A.NRows(), B.NRows())), minNC(min(A.NCols(), B.NCols())) ;
	Diff.resize(minNR, minNC) ;
	for (int i(0); i<minNR; i++)
		for(int j(0); j<minNC; j++)
			Diff._V[i][j] = A._V[i][j] - B._V[i][j] ;
}
#endif //_DATAMAT_H_
